import pandas as pd
import logging

__all__ = ['reduce_mem_usage','reduce_mem_usage_category']

import numpy as np
import pandas as pd

def reduce_mem_usage(df):
    """Reduces Memory usage of a DataFrame, converts strings to "object"

    Args:
        df ([Dataframe]): [input dataframe]

    Returns:
        df ([Dataframe]): [output dataframe]
    """
    start_mem = df.memory_usage().sum() / 1024**2
    logging.info('Memory usage of dataframe is {:.2f} MB'.format(start_mem))

    for col in df.columns:
        col_type = df[col].dtype

        if col_type != object:
            c_min = df[col].min()
            c_max = df[col].max()
            if str(col_type)[:3] == 'int':
                if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                    df[col] = df[col].astype(np.int8)
                elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                    df[col] = df[col].astype(np.int16)
                elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                    df[col] = df[col].astype(np.int32)
                elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                    df[col] = df[col].astype(np.int64)
            else:
                if c_min > np.finfo(np.float16).min and c_max < np.finfo(np.float16).max:
                    df[col] = df[col].astype(np.float16)
                elif c_min > np.finfo(np.float32).min and c_max < np.finfo(np.float32).max:
                    df[col] = df[col].astype(np.float32)
                else:
                    df[col] = df[col].astype(np.float64)
        else:
            df[col] = df[col].astype('object')

    end_mem = df.memory_usage().sum() / 1024**2
    logging.info(
        'Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    logging.info('Decreased by {:.1f}%'.format(
        100 * (start_mem - end_mem) / start_mem))

    return df


def reduce_mem_usage_category(df):
    """Reduces Memory usage of a DataFrame, converts string columns to Category, which needs less memory)"

    Args:
        df ([Dataframe]): [input dataframe]

    Returns:
        df ([Dataframe]): [output dataframe]
    """
    start_mem = df.memory_usage().sum() / 1024**2
    logging.info('Memory usage of dataframe is {:.2f} MB'.format(start_mem))

    for col in df.columns:
        col_type = df[col].dtype

        if col_type == object:
            df[col] = df[col].astype('category')

    end_mem = df.memory_usage().sum() / 1024**2
    logging.info(
        'Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    logging.info('Decreased by {:.1f}%'.format(
        100 * (start_mem - end_mem) / start_mem))
    return df